__title__ = 'eciespy'
__version__ = '0.1.2'
__description__ = 'Elliptic Curve Integrated Encryption Scheme for secp256k1 in Python'
__url__ = 'https://github.com/kigawas/eciespy'
__author__ = 'Weiliang Li'
__author_email__ = 'to.be.impressive@gmail.com'
__license__ = 'Apache-2.0'
